function pq_norms_val = update_pq_norms(mask_labeled, pq_norms_val, i_p, signal_params, dgt, idgt)


%% pq_norms_val = update_pq_norms(mask_labeled, pq_norms_val, i_p, signal_params, dgt, idgt)
%
% This function compute  bit faster norms between two Gabor multiplier.
%  This is an improvement of the function get_pq_norms.m
%
% Inputs:
%     - mask_labeled: mask labeled-mask with P regions
%     -pq_norms_val: array with all norms between all p and q regions
%     -i_p: ineteger
%     -signal_params: Signals parameters(sig_len, fs)
%     - dgt, idgt: Discrete Gabor Transform parameters(hop, nbins,win, ect..)
% Output:
%     - pq_norms_val :  array with all norms between all p and q regions
%
% Author: Marina KREME

%%


sig_len = signal_params.sig_len;
n_labels = size(pq_norms_val,1);

mask_p = (mask_labeled==i_p);
gabmul_p = gen_gabmul_operator(dgt, idgt, mask_p);

for i_q =1:n_labels
    if i_p == i_q
        continue
    end
    mask_q = (mask_labeled==i_q);
    gabmul_q = gen_gabmul_operator(dgt, idgt, mask_q);
    
    gabmul_pq =@(x)gabmul_p(gabmul_q(x));
    gabmul_pq_norm =  real(eigs(gabmul_pq, sig_len,1));
end
if i_q < i_p
    pq_norms_val(i_p, i_q) = gabmul_pq_norm;
else
    pq_norms_val(i_q, i_p) = gabmul_pq_norm;
end


end

